@@ -33,10 +33,9 @@
 #include "http_log.h"
 #include "http_protocol.h"
 
-DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status,
-                                      int error_id, const char *desc)
+DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status, int error_id,
+                                      apr_status_t aprerr, const char *desc)
 {
-    int save_errno = errno;
     dav_error *err = apr_pcalloc(p, sizeof(*err));
 
     /* DBG3("dav_new_error: %d %d %s", status, error_id, desc ? desc : "(no desc)"); */
@@ -44,17 +43,18 @@ DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status,
     err->status = status;
     err->error_id = error_id;
     err->desc = desc;
-    err->save_errno = save_errno;
+    err->aprerr = aprerr;
 
     return err;
 }
 
 DAV_DECLARE(dav_error*) dav_new_error_tag(apr_pool_t *p, int status,
-                                          int error_id, const char *desc,
+                                          int error_id, apr_status_t aprerr,
+                                          const char *desc,
                                           const char *namespace,
                                           const char *tagname)
 {
-    dav_error *err = dav_new_error(p, status, error_id, desc);
+    dav_error *err = dav_new_error(p, status, error_id, aprerr, desc);
 
     err->tagname = tagname;
     err->namespace = namespace;
@@ -77,6 +77,30 @@ DAV_DECLARE(dav_error*) dav_push_error(apr_pool_t *p, int status,
     return err;
 }
 
+DAV_DECLARE(dav_error*) dav_join_error(dav_error *dest, dav_error *src)
+{
+    dav_error *curr = dest;
+
+    /* src error doesn't exist so nothing to join just return dest */
+    if (src == NULL) {
+        return dest;
+    }
+
+    /* dest error doesn't exist so nothing to join just return src */
+    if (curr == NULL) {
+        return src;
+    }
+
+    /* find last error in dest stack */
+    while (curr->prev != NULL) {
+        curr = curr->prev;
+    }
+
+    /* add the src error onto end of dest stack and return it */
+    curr->prev = src;
+    return dest;
+}
+
 DAV_DECLARE(void) dav_check_bufsize(apr_pool_t * p, dav_buffer *pbuf,
                                     apr_size_t extra_needed)
 {
@@ -372,8 +396,10 @@ DAV_DECLARE(const char *) dav_xml_get_cdata(const apr_xml_elem *elem, apr_pool_t
 
     if (strip_white) {
         /* trim leading whitespace */
-        while (apr_isspace(*cdata))     /* assume: return false for '\0' */
+        while (apr_isspace(*cdata)) {     /* assume: return false for '\0' */
             ++cdata;
+            --len;
+        }
 
         /* trim trailing whitespace */
         while (len-- > 0 && apr_isspace(cdata[len]))
@@ -402,7 +428,7 @@ DAV_DECLARE(void) dav_xmlns_add(dav_xmlns_info *xi,
     apr_hash_set(xi->prefix_uri, prefix, APR_HASH_KEY_STRING, uri);
 
     /* note: this may overwrite an existing URI->prefix mapping, but it
-       doesn't matter -- any prefix is usuable to specify the URI. */
+       doesn't matter -- any prefix is usable to specify the URI. */
     apr_hash_set(xi->uri_prefix, uri, APR_HASH_KEY_STRING, prefix);
 }
 
@@ -444,8 +470,8 @@ DAV_DECLARE(void) dav_xmlns_generate(dav_xmlns_info *xi,
 
         apr_hash_this(hi, &prefix, NULL, &uri);
 
-        s = apr_psprintf(xi->pool, " xmlns:%s=\"%s\"",
-                         (const char *)prefix, (const char *)uri);
+        s = apr_pstrcat(xi->pool, " xmlns:", (const char *)prefix, "=\"",
+                        (const char *)uri, "\"", NULL);
         apr_text_append(xi->pool, phdr, s);
     }
 }
@@ -561,7 +587,7 @@ static dav_error * dav_add_if_state(apr_pool_t *p, dav_if_header *ih,
 }
 
 /* fetch_next_token returns the substring from str+1
- * to the next occurence of char term, or \0, whichever
+ * to the next occurrence of char term, or \0, whichever
  * occurs first.  Leading whitespace is ignored.
  */
 static char *dav_fetch_next_token(char **str, char term)
@@ -599,6 +625,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
     const char *state_token;
     const char *uri = NULL;        /* scope of current production; NULL=no-tag */
     apr_size_t uri_len = 0;
+    apr_status_t rv;
     dav_if_header *ih = NULL;
     apr_uri_t parsed_uri;
     const dav_hooks_locks *locks_hooks = DAV_GET_HOOKS_LOCKS(r);
@@ -617,26 +644,36 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
             if (list_type == no_tagged
                 || ((uri = dav_fetch_next_token(&str, '>')) == NULL)) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_TAGGED,
+                                     DAV_ERR_IF_TAGGED, 0,
                                      "Invalid If-header: unclosed \"<\" or "
                                      "unexpected tagged-list production.");
             }
 
             /* 2518 specifies this must be an absolute URI; just take the
              * relative part for later comparison against r->uri */
-            if (apr_uri_parse(r->pool, uri, &parsed_uri) != APR_SUCCESS
+            if ((rv = apr_uri_parse(r->pool, uri, &parsed_uri)) != APR_SUCCESS
                 || !parsed_uri.path) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_TAGGED,
+                                     DAV_ERR_IF_TAGGED, rv,
                                      "Invalid URI in tagged If-header.");
             }
             /* note that parsed_uri.path is allocated; we can trash it */
 
             /* clean up the URI a bit */
             ap_getparents(parsed_uri.path);
+
+            /* the resources we will compare to have unencoded paths */
+            if (ap_unescape_url(parsed_uri.path) != OK) {
+                return dav_new_error(r->pool, HTTP_BAD_REQUEST,
+                                     DAV_ERR_IF_TAGGED, rv,
+                                     "Invalid percent encoded URI in "
+                                     "tagged If-header.");
+            }
+
             uri_len = strlen(parsed_uri.path);
-            if (uri_len > 1 && parsed_uri.path[uri_len - 1] == '/')
+            if (uri_len > 1 && parsed_uri.path[uri_len - 1] == '/') {
                 parsed_uri.path[--uri_len] = '\0';
+            }
 
             uri = parsed_uri.path;
             list_type = tagged;
@@ -651,14 +688,14 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
             if ((list = dav_fetch_next_token(&str, ')')) == NULL) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_UNCLOSED_PAREN,
+                                     DAV_ERR_IF_UNCLOSED_PAREN, 0,
                                      "Invalid If-header: unclosed \"(\".");
             }
 
             if ((ih = dav_add_if_resource(r->pool, ih, uri, uri_len)) == NULL) {
                 /* ### dav_add_if_resource() should return an error for us! */
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_PARSE,
+                                     DAV_ERR_IF_PARSE, 0,
                                      "Internal server error parsing \"If:\" "
                                      "header.");
             }
@@ -673,7 +710,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if ((state_token = dav_fetch_next_token(&list, '>')) == NULL) {
                         /* ### add a description to this error */
                         return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                             DAV_ERR_IF_PARSE, NULL);
+                                             DAV_ERR_IF_PARSE, 0, NULL);
                     }
 
                     if ((err = dav_add_if_state(r->pool, ih, state_token, dav_if_opaquelock,
@@ -688,7 +725,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if ((state_token = dav_fetch_next_token(&list, ']')) == NULL) {
                         /* ### add a description to this error */
                         return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                             DAV_ERR_IF_PARSE, NULL);
+                                             DAV_ERR_IF_PARSE, 0, NULL);
                     }
 
                     if ((err = dav_add_if_state(r->pool, ih, state_token, dav_if_etag,
@@ -703,7 +740,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if (list[1] == 'o' && list[2] == 't') {
                         if (condition != DAV_IF_COND_NORMAL) {
                             return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                                 DAV_ERR_IF_MULTIPLE_NOT,
+                                                 DAV_ERR_IF_MULTIPLE_NOT, 0,
                                                  "Invalid \"If:\" header: "
                                                  "Multiple \"not\" entries "
                                                  "for the same state.");
@@ -719,7 +756,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
                 default:
                     return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                         DAV_ERR_IF_UNK_CHAR,
+                                         DAV_ERR_IF_UNK_CHAR, 0,
                                          apr_psprintf(r->pool,
                                                      "Invalid \"If:\" "
                                                      "header: Unexpected "
@@ -738,7 +775,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
         default:
             return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                 DAV_ERR_IF_UNK_CHAR,
+                                 DAV_ERR_IF_UNK_CHAR, 0,
                                  apr_psprintf(r->pool,
                                              "Invalid \"If:\" header: "
                                              "Unexpected character "
@@ -884,7 +921,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     */
     if (flags & DAV_LOCKSCOPE_EXCLUSIVE) {
         if (lock_list != NULL) {
-            return dav_new_error(p, HTTP_LOCKED, 0,
+            return dav_new_error(p, HTTP_LOCKED, 0, 0,
                                  "Existing lock(s) on the requested resource "
                                  "prevent an exclusive lock.");
         }
@@ -902,7 +939,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         */
         for (lock = lock_list; lock != NULL; lock = lock->next) {
             if (lock->scope == DAV_LOCKSCOPE_EXCLUSIVE) {
-                return dav_new_error(p, HTTP_LOCKED, 0,
+                return dav_new_error(p, HTTP_LOCKED, 0, 0,
                                      "The requested resource is already "
                                      "locked exclusively.");
             }
@@ -919,13 +956,16 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         /*
         ** For methods other than LOCK:
         **
-        ** If we have no locks, then <seen_locktoken> can be set to true --
+        ** If we have no locks or if the resource is not being modified
+        ** (per RFC 4918 the lock token is not required on resources
+        ** we are not changing), then <seen_locktoken> can be set to true --
         ** pretending that we've already met the requirement of seeing one
         ** of the resource's locks in the If: header.
         **
         ** Otherwise, it must be cleared and we'll look for one.
         */
-        seen_locktoken = (lock_list == NULL);
+        seen_locktoken = (lock_list == NULL
+                          || flags & DAV_VALIDATE_NO_MODIFY);
     }
 
     /*
@@ -941,7 +981,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         if (seen_locktoken)
             return NULL;
 
-        return dav_new_error(p, HTTP_LOCKED, 0,
+        return dav_new_error(p, HTTP_LOCKED, 0, 0,
                              "This resource is locked and an \"If:\" header "
                              "was not supplied to allow access to the "
                              "resource.");
@@ -965,7 +1005,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     if (lock_list == NULL && if_header->dummy_header) {
         if (flags & DAV_VALIDATE_IS_PARENT)
             return NULL;
-        return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+        return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                              "The locktoken specified in the \"Lock-Token:\" "
                              "header is invalid because this resource has no "
                              "outstanding locks.");
@@ -1171,7 +1211,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
                                             "\" submitted a locktoken created "
                                             "by user \"",
                                             lock->auth_user, "\".", NULL);
-                        return dav_new_error(p, HTTP_FORBIDDEN, 0, errmsg);
+                        return dav_new_error(p, HTTP_FORBIDDEN, 0, 0, errmsg);
                     }
 
                     /*
@@ -1291,7 +1331,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
                 return NULL;
             }
 
-            return dav_new_error(p, HTTP_LOCKED, 0 /* error_id */,
+            return dav_new_error(p, HTTP_LOCKED, 0 /* error_id */, 0,
                                  "This resource is locked and the \"If:\" "
                                  "header did not specify one of the "
                                  "locktokens for this resource's lock(s).");
@@ -1305,19 +1345,19 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         ** bad Lock-Token first. That is considered a 400 (Bad Request).
         */
         if (if_header->dummy_header) {
-            return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+            return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                                  "The locktoken specified in the "
                                  "\"Lock-Token:\" header did not specify one "
                                  "of this resource's locktoken(s).");
         }
 
         if (reason == NULL) {
-            return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0,
+            return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0, 0,
                                  "The preconditions specified by the \"If:\" "
                                  "header did not match this resource.");
         }
 
-        return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0,
+        return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0, 0,
                              apr_psprintf(p,
                                          "The precondition(s) specified by "
                                          "the \"If:\" header did not match "
@@ -1362,13 +1402,13 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     ** We want to note the 400 (Bad Request) in favor of a 423 (Locked).
     */
     if (if_header->dummy_header) {
-        return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+        return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                              "The locktoken specified in the "
                              "\"Lock-Token:\" header did not specify one "
                              "of this resource's locktoken(s).");
     }
 
-    return dav_new_error(p, HTTP_LOCKED, 1 /* error_id */,
+    return dav_new_error(p, HTTP_LOCKED, 1 /* error_id */, 0,
                          "This resource is locked and the \"If:\" header "
                          "did not specify one of the "
                          "locktokens for this resource's lock(s).");
@@ -1420,11 +1460,11 @@ static int dav_meets_conditions(request_rec *r, int resource_state)
 
     retVal = ap_meets_conditions(r);
 
-    /* If-None-Match '*' fix. If-None-Match '*' request should succeed 
+    /* If-None-Match '*' fix. If-None-Match '*' request should succeed
      * if the resource does not exist. */
     if (retVal == HTTP_PRECONDITION_FAILED) {
         /* Note. If if_none_match != NULL, if_none_match is the culprit.
-         * Since, in presence of If-None-Match, 
+         * Since, in presence of If-None-Match,
          * other If-* headers are undefined. */
         if ((if_none_match =
             apr_table_get(r->headers_in, "If-None-Match")) != NULL) {
@@ -1477,7 +1517,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         ** ### bleck. we can't return errors for other URIs unless we have
         ** ### a "response" ptr.
         */
-        return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0,
+        return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0, 0,
                              "DESIGN ERROR: dav_validate_request called "
                              "with depth>0, but no response ptr.");
     }
@@ -1509,7 +1549,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         apr_table_unset(r->headers_out, "ETag");
     }
     if (result != OK) {
-        return dav_new_error(r->pool, result, 0, NULL);
+        return dav_new_error(r->pool, result, 0, 0, NULL);
     }
 
     /* always parse (and later process) the If: header */
@@ -1555,8 +1595,10 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         }
     }
 
-    /* (1) Validate the specified resource, at the specified depth */
-    if (resource->exists && depth > 0) {
+    /* (1) Validate the specified resource, at the specified depth.
+     * Avoid the walk there is no if_header and we aren't planning
+     * to modify this resource. */
+    if (resource->exists && depth > 0 && !(!if_header && flags & DAV_VALIDATE_NO_MODIFY)) {
         dav_walker_ctx ctx = { { 0 } };
         dav_response *multi_status;
 
@@ -1577,7 +1619,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
 
         err = (*repos_hooks->walk)(&ctx.w, DAV_INFINITY, &multi_status);
         if (err == NULL) {
-            *response = multi_status;;
+            *response = multi_status;
         }
         /* else: implies a 5xx status code occurred. */
     }
@@ -1593,7 +1635,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         err = (*repos_hooks->get_parent_resource)(resource, &parent_resource);
 
         if (err == NULL && parent_resource == NULL) {
-            err = dav_new_error(r->pool, HTTP_FORBIDDEN, 0,
+            err = dav_new_error(r->pool, HTTP_FORBIDDEN, 0, 0,
                                 "Cannot access parent of repository root.");
         }
         else if (err == NULL) {
@@ -1649,7 +1691,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
 
         if ((flags & DAV_VALIDATE_USE_424) != 0) {
             /* manufacture a 424 error to hold the multistatus response(s) */
-            return dav_new_error(r->pool, HTTP_FAILED_DEPENDENCY, 0,
+            return dav_new_error(r->pool, HTTP_FAILED_DEPENDENCY, 0, 0,
                                  "An error occurred on another resource, "
                                  "preventing the requested operation on "
                                  "this resource.");
@@ -1684,7 +1726,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         *response = new_response;
 
         /* manufacture a 207 error for the multistatus response(s) */
-        return dav_new_error(r->pool, HTTP_MULTI_STATUS, 0,
+        return dav_new_error(r->pool, HTTP_MULTI_STATUS, 0, 0,
                              "Error(s) occurred on resources during the "
                              "validation process.");
     }
@@ -1713,7 +1755,7 @@ DAV_DECLARE(dav_error *) dav_get_locktoken_list(request_rec *r,
     }
 
     while (if_header != NULL) {
-        if_state = if_header->state;        /* Begining of the if_state linked list */
+        if_state = if_header->state;        /* Beginning of the if_state linked list */
         while (if_state != NULL)        {
             if (if_state->condition == DAV_IF_COND_NORMAL
                 && if_state->type == dav_if_opaquelock) {
@@ -1728,7 +1770,7 @@ DAV_DECLARE(dav_error *) dav_get_locktoken_list(request_rec *r,
     }
     if (*ltl == NULL) {
         /* No nodes added */
-        return dav_new_error(r->pool, HTTP_BAD_REQUEST, DAV_ERR_IF_ABSENT,
+        return dav_new_error(r->pool, HTTP_BAD_REQUEST, DAV_ERR_IF_ABSENT, 0,
                              "No locktokens were specified in the \"If:\" "
                              "header, so the refresh could not be performed.");
     }
@@ -1781,10 +1823,11 @@ DAV_DECLARE(void) dav_add_vary_header(request_rec *in_req,
      * so only do this check if there is a versioning provider */
     if (vsn_hooks != NULL) {
         const char *target = apr_table_get(in_req->headers_in, DAV_LABEL_HDR);
-        const char *vary = apr_table_get(out_req->headers_out, "Vary");
 
         /* If Target-Selector specified, add it to the Vary header */
         if (target != NULL) {
+            const char *vary = apr_table_get(out_req->headers_out, "Vary");
+
             if (vary == NULL)
                 vary = DAV_LABEL_HDR;
             else
@@ -1825,7 +1868,7 @@ static dav_error * dav_can_auto_checkout(
             const dav_hooks_locks *locks_hooks = DAV_GET_HOOKS_LOCKS(r);
 
             if (locks_hooks == NULL) {
-                return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0,
+                return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0, 0,
                                      "Auto-checkout is only enabled for locked resources, "
                                      "but there is no lock provider.");
             }
@@ -1880,7 +1923,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
             goto done;
 
         if (parent == NULL || !parent->exists) {
-            err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+            err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                 apr_psprintf(r->pool,
                                             "Missing one or more intermediate "
                                             "collections. Cannot create resource %s.",
@@ -1902,7 +1945,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
             }
 
             if (!checkout_parent) {
-                err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+                err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                     "<DAV:cannot-modify-checked-in-parent>");
                 goto done;
             }
@@ -1961,7 +2004,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
         }
 
         if (!checkout_resource) {
-            err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+            err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                 "<DAV:cannot-modify-version-controlled-content>");
             goto done;
         }
